<?php
/*
Plugin Name: Conference Scheduler
Plugin URI: http://myceliumdesign.ca/conf_scheduler/
Description: Display and organize your conference workshops in a powerful, easy-to-use system.
Version: 1.1
Author: Shane Warner
Author URI: http://myceliumdesign.ca/
Text Domain: conf_scheduler
Domain Path: /languages
License: GPLv2 or later
License URI: http://www.gnu.org/licenses/gpl-2.0.html

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

if( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if (!defined('CONF_SCHEDULER_VERSION'))
    define('CONF_SCHEDULER_VERSION', '1.1');

if( ! class_exists('conf_scheduler') ) :

class conf_scheduler {

  public $last_date = '0';
	public $message = '';
  public $options = array();

	function initialize() {
    $this->options = get_option('conf_scheduler_options', array());

		add_action('init', array($this, 'conf_create_post_types'), 10 );
		add_action('init', array($this, 'activiation_check'), 15);
    add_action('plugins_loaded', array($this, 'translate'));
    add_action('conf_sessions_add_form_fields', array($this, 'sessions_meta_box_html'));
    add_action('conf_sessions_edit_form_fields', array($this, 'sessions_edit_meta_box_html'), 100);
		add_filter('manage_edit-conf_sessions_columns', array($this, 'sessions_custom_columns_head'));
		add_filter('manage_conf_sessions_custom_column', array($this, 'sessions_custom_columns_content'), 10, 3);
		add_filter('manage_edit-conf_sessions_sortable_columns',array($this, 'sessions_sortable_columns'), 10);

    add_filter('manage_conf_workshop_posts_columns', array($this, 'workshops_custom_columns_head'));
    add_filter('manage_conf_workshop_posts_custom_column', array($this, 'workshops_custom_columns_content'), 10, 2);
    //add_filter('manage_edit-conf_workshop_sortable_columns',array($this, 'workshop_sortable_columns'), 10);
		add_action('pre_get_terms', array($this, 'sessions_admin_orderby') );
    //add_action('pre_get_posts', array($this, 'workshops_admin_orderby') );
    //add_action('posts_clauses', array($this, 'workshops_admin_orderby'), 10, 2);
    add_action('save_post_conf_workshop', array($this, 'save_workshop_meta'), 10, 2 );
    add_action('create_conf_sessions', array($this, 'save_session_meta'), 10, 2 );
    add_action('edit_conf_sessions', array($this, 'save_session_meta'), 10, 2 );

		add_action('admin_notices', array($this, 'admin_notice_success' ));
    add_action( 'admin_footer', array($this, 'conf_scheduler_info_html' ));
    add_action( 'admin_enqueue_scripts', array($this, 'load_admin_style' ) );
    add_action( 'admin_menu', array($this, 'options_menu_item') );

		add_shortcode('conf_scheduler', array($this, 'render_schedule'));
    add_action( 'wp_ajax_conf_scheduler_register', array($this, 'ajax_registration_callback') );
    add_action( 'wp_ajax_conf_scheduler_list_participants', array($this, 'list_registered_participants') );

		wp_register_style('conf_scheduler', plugin_dir_url( __FILE__ ) .'css/conf_scheduler.css' );
    wp_register_style('conf_scheduler_admin-css', plugin_dir_url( __FILE__ ) .'css/conf_scheduler_admin.css' );
    wp_register_style('select2v4-css', plugin_dir_url( __FILE__ ) .'css/select2.min.css' );
    wp_register_style('datetime_picker-css', plugin_dir_url( __FILE__ ) .'css/bootstrap_datetime_picker.css' );
		wp_register_style( 'fontawesome', '//maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css', array(), '4.7' );
		wp_register_script('isotope', plugin_dir_url( __FILE__ ) .'js/isotope.pkgd.min.js', array( 'jquery' ), null);
		wp_register_script('js-cookie', plugin_dir_url( __FILE__ ) .'js/js.cookie.js' );
		wp_register_script('conf_scheduler', plugin_dir_url( __FILE__ ) .'js/conf_scheduler.js' );
    wp_register_script('conf_scheduler_admin-js', plugin_dir_url( __FILE__ ) .'js/admin.js' );
    wp_register_script('select2v4-js', plugin_dir_url( __FILE__ ) .'js/select2.full.min.js' );
    wp_register_script('datetime_picker-js', plugin_dir_url( __FILE__ ) .'js/bootstrap_datetime_picker.min.js' );
	}

  function translate() {
    load_plugin_textdomain( 'conf_scheduler', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
  }

	// Add support for prorgams
	function conf_create_post_types() {

		// Add Sessions taxonomy, make it hierarchical
		$labels = array(
			'name'              => _x( 'Sessions', 'taxonomy general name' ),
			'singular_name'     => _x( 'Session', 'taxonomy singular name' ),
			'search_items'      => __( 'Search Sessions' ),
			'all_items'         => __( 'All Sessions' ),
			'parent_item'       => __( 'Parent Session' ),
			'parent_item_colon' => __( 'Parent Session:' ),
			'edit_item'         => __( 'Edit Session' ),
			'update_item'       => __( 'Update Session' ),
			'add_new_item'      => __( 'Add New Session' ),
			'new_item_name'     => __( 'New Session Name' ),
			'menu_name'         => __( 'Sessions' ),
		);

		$args = array(
			'hierarchical'      => true,
			'labels'            => $labels,
			'show_ui'           => true,
			'show_admin_column' => true,
			'query_var'         => true,
			'rewrite'           => array( 'slug' => 'sessions' )
		);

		register_taxonomy( 'conf_sessions', array( 'conf_workshop' ), $args );

		// Add Streams, make it hierarchical
		$labels = array(
			'name'              => _x( 'Streams', 'taxonomy general name' ),
			'singular_name'     => _x( 'Stream', 'taxonomy singular name' ),
			'search_items'      => __( 'Search Streams' ),
			'all_items'         => __( 'All Streams' ),
			'parent_item'       => __( 'Parent Stream' ),
			'parent_item_colon' => __( 'Parent Stream:' ),
			'edit_item'         => __( 'Edit Stream' ),
			'update_item'       => __( 'Update Stream' ),
			'add_new_item'      => __( 'Add New Stream' ),
			'new_item_name'     => __( 'New Stream Name' ),
			'menu_name'         => __( 'Streams' ),
		);

		$args = array(
			'hierarchical'      => true,
			'labels'            => $labels,
			'show_ui'           => true,
			'show_admin_column' => true,
			'query_var'         => true,
			'rewrite'           => array( 'slug' => 'streams' ),
		);

		register_taxonomy( 'conf_streams', array( 'conf_workshop' ), $args );

    // Add Areas taxonomy
    $labels = array(
      'name'              => _x( 'Areas', 'taxonomy general name' ),
      'singular_name'     => _x( 'Area', 'taxonomy singular name' ),
      'search_items'      => __( 'Search Areas' ),
      'all_items'         => __( 'All Areas' ),
      'parent_item'       => __( 'Parent Area' ),
      'parent_item_colon' => __( 'Parent Area:' ),
      'edit_item'         => __( 'Edit Area' ),
      'update_item'       => __( 'Update Area' ),
      'add_new_item'      => __( 'Add New Area' ),
      'new_item_name'     => __( 'New Area Name' ),
      'menu_name'         => __( 'Areas' ),
    );

    $args = array(
      'hierarchical'      => false,
      'labels'            => $labels,
      'show_ui'           => true,
      'show_admin_column' => true,
      'query_var'         => true,
      'rewrite'           => array( 'slug' => 'areas' ),
    );

    register_taxonomy( 'conf_areas', array( 'conf_workshop' ), $args );

		// Register Workshops
		$labels = array(
	    'name'               => 'Workshops',
	    'singular_name'      => 'Workshop',
	    'add_new'            => 'Add New',
	    'add_new_item'       => 'Add New Workshop',
	    'edit_item'          => 'Edit Workshop',
	    'new_item'           => 'New Workshop',
	    'all_items'          => 'Workshops',
	    'view_item'          => 'View Workshop',
	    'search_items'       => 'Search Workshops',
	    'not_found'          => 'No workshops found',
	    'not_found_in_trash' => 'No workshops found in Trash',
	    'parent_item_colon'  => '',
	    'menu_name'          => 'Conference'
		);
		$args = array(
		'labels' => $labels,
		'public' => true,
		'exclude_from_search' => false,
		'taxonomies'=> array('conf_sessions', 'conf_streams', 'conf_areas'),
		'supports' =>  array('thumbnail', 'title', 'editor'),
		'menu_position' => 20,
    'register_meta_box_cb' => array($this, 'workshop_meta_boxes'),
    'menu_icon' => 'data:image/svg+xml;base64,' . base64_encode( '<svg version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" width="20px" height="20px" viewBox="0 0 80 80" xml:space="preserve">
<path fill="#a0a5aa" d="M35,2.5h10v20H35V2.5z M26.094,38.594c0-1.667-0.599-3.099-1.797-4.297S21.667,32.5,20,32.5s-3.099,0.599-4.297,1.797s-1.797,2.63-1.797,4.297s0.599,3.073,1.797,4.219s2.63,1.719,4.297,1.719s3.099-0.573,4.297-1.719S26.094,40.261,26.094,38.594z M30,60.312c0-3.75-0.963-6.901-2.891-9.453s-4.245-3.828-6.953-3.828s-5.026,1.276-6.953,3.828s-2.891,5.703-2.891,9.453c0,2.292,1.641,3.906,4.922,4.844s6.562,0.938,9.844,0S30,62.604,30,60.312z M37.5,57.5c-0.729,0-1.328,0.234-1.797,0.703S35,59.271,35,60s0.234,1.328,0.703,1.797S36.771,62.5,37.5,62.5h30.469c0.625,0,1.198-0.234,1.719-0.703s0.781-1.067,0.781-1.797s-0.26-1.328-0.781-1.797S68.594,57.5,67.969,57.5H37.5z M37.5,47.5c-0.729,0-1.328,0.234-1.797,0.703S35,49.271,35,50s0.234,1.328,0.703,1.797S36.771,52.5,37.5,52.5h30.469c0.625,0,1.198-0.234,1.719-0.703s0.781-1.067,0.781-1.797s-0.26-1.328-0.781-1.797S68.594,47.5,67.969,47.5H37.5z M37.5,37.5c-0.729,0-1.328,0.234-1.797,0.703S35,39.271,35,40s0.234,1.328,0.703,1.797S36.771,42.5,37.5,42.5h30.469c0.625,0,1.198-0.234,1.719-0.703s0.781-1.067,0.781-1.797s-0.26-1.328-0.781-1.797S68.594,37.5,67.969,37.5H37.5z M75,12.5c1.354,0,2.526,0.495,3.516,1.484S80,16.146,80,17.5v55c0,1.354-0.495,2.526-1.484,3.516S76.354,77.5,75,77.5H5c-1.354,0-2.526-0.495-3.516-1.484S0,73.854,0,72.5v-55c0-1.354,0.495-2.526,1.484-3.516S3.646,12.5,5,12.5h25v5c-1.354,0-2.526,0.495-3.516,1.484S25,21.146,25,22.5s0.495,2.526,1.484,3.516S28.646,27.5,30,27.5h5h10h5c1.354,0,2.526-0.495,3.516-1.484S55,23.854,55,22.5s-0.495-2.526-1.484-3.516S51.354,17.5,50,17.5v-5H75z"/>
</svg>')
		);

		register_post_type( 'conf_workshop', $args );

	}

	// upgrade check
	function activiation_check() {
    $installed = get_option('conf_scheduler_version');
		if ($installed && CONF_SCHEDULER_VERSION != $installed)
    $this->do_upgrade();

    if(!$installed)
      update_option('conf_scheduler_version', CONF_SCHEDULER_VERSION);
	}

	// do upgrade
	function do_upgrade() {

	  update_option('conf_scheduler_version', CONF_SCHEDULER_VERSION);
	}

	function admin_notice_success() {
		if ($this->message != '') {
	    echo '<div class="notice notice-success is-dismissible"><p>'.$this->message.'</p></div>';
		}
	}

  function sessions_meta_box_html() {
    wp_enqueue_script('datetime_picker-js');
    wp_enqueue_style('datetime_picker-css');
    wp_enqueue_script('conf_scheduler_admin-js');
    wp_enqueue_style('conf_scheduler_admin-css');

    wp_nonce_field( basename( __FILE__ ), 'conf_scheduler_sessions' );
    ?>

    <div class="form-field">
      <label for="start"><?php _e('Start date/time','conf_scheduler')?>:</label>
      <div class="datetime-field-wrap"><input type='text' name='start' id='start' value=''/><label for="start"><span class="dashicons dashicons-calendar-alt"></span></label></div>
    </div>

    <div class="form-field">
      <label for="length"><?php _e('Length','conf_scheduler')?>:</label>
      <input type='number' name='length' id='length' value=''/>
      <p><?php _e('Length of session in minutes.','conf_scheduler')?></p>
    </div>

    <div class="form-field collapse-wrap">
  		<label for="collapse"><?php _e('Show Sub-sessions','conf_scheduler')?></label>
      <ul>
        <li><label><input type="radio" id="collapse-0" name="collapse" value="seperate"><?php _e('Seperate','conf_scheduler')?></label></li>
        <li><label><input type="radio" id="collapse-1" name="collapse" value="collapse"><?php _e('Collapsed','conf_scheduler')?></label></li>
      </ul>
      <p><?php _e('Show sub-sessions collapsed in with this session, or show them as separate sessions.','conf_scheduler')?></p>
    </div>

    <?php
  }

  function sessions_edit_meta_box_html() {
    global $tag;
    wp_enqueue_script('datetime_picker-js');
    wp_enqueue_style('datetime_picker-css');
    wp_enqueue_script('conf_scheduler_admin-js');
    wp_enqueue_style('conf_scheduler_admin-css');

    $fields = array('start', 'length', 'collapse');
    $values = array();

    foreach ($fields as $field) {
      $values[$field] = get_term_meta( $tag->term_id, $field, true );
    }

    // reformat time for datetime field
    $values['start'] = date('Y-m-d g:i a', strtotime($values['start']));

    wp_nonce_field( basename( __FILE__ ), 'conf_scheduler_sessions' );
    ?>
    <tr class="form-field">
      <th scope="row"><label for="start"><?php _e('Start date/time','conf_scheduler')?>:</label></th>
      <td><div class="datetime-field-wrap"><input type='text' name='start' id='start' value='<?php echo esc_attr($values['start']);?>' data-date='<?php echo esc_attr($values['start']);?>'/><label for="start"><span class="dashicons dashicons-calendar-alt"></span></label></div></td>
    </tr>

    <tr class="form-field">
      <th scope="row"><label for="length"><?php _e('Length','conf_scheduler')?>:</label></th>
      <td><input type='number' name='length' id='length' value='<?php echo esc_attr($values['length']);?>'/>
      <p class="description"><?php _e('Length of session in minutes.','conf_scheduler')?></p></td>
    </div>

    <tr class="form-field collapse-wrap">
  		<th scope="row"><label for="collapse"><?php _e('Show Sub-sessions','conf_scheduler')?></label></th>
      <td><ul>
        <li><label><input type="radio" id="collapse-0" name="collapse" value="seperate"<?php if($values['collapse']!='collapse') echo ' checked="checked"';?>><?php _e('Seperate','conf_scheduler')?></label></li>
        <li><label><input type="radio" id="collapse-1" name="collapse" value="collapse"<?php if($values['collapse']=='collapse') echo ' checked="checked"';?>><?php _e('Collapsed','conf_scheduler')?></label></li>
      </ul>
      <p class="description"><?php _e('Show sub-sessions collapsed in with this session, or show them as separate sessions.','conf_scheduler')?></p>
    </td>
  </tr>

    <?php
  }

  function workshop_meta_boxes() {
    add_meta_box(
  		'workshop_meta_box',
  		'Workshop Details',
  		array($this, 'workshop_meta_box_html'),
  		'conf_workshop',
  		'normal',
  		'high'
  	);
    add_meta_box(
  		'workshop_side_meta_box',
  		'Workshop Info',
  		array($this, 'workshop_side_meta_box_html'),
  		'conf_workshop',
  		'side',
  		'high'
  	);
  }

  function workshop_meta_box_html() {
    global $post;

    wp_enqueue_media();
    wp_enqueue_script('conf_scheduler_admin-js');
    wp_enqueue_style('conf_scheduler_admin-css');

  	wp_nonce_field( basename( __FILE__ ), 'conf_scheduler' );
  	$fields = array('presenter_bio', 'file_attachments_id');
    $values = array();

    foreach ($fields as $field) {
      $values[$field] = get_post_meta( $post->ID, $field, true );
    }

    $values['file_attachments_id'] = $values['file_attachments_id'] ? $values['file_attachments_id'] : '';

    ?>
    <label for="presenter_bio"><?php _e('Presenter Bio','conf_scheduler')?>:</label>
    <textarea name="presenter_bio" id="presenter_bio"><?php echo esc_textarea($values['presenter_bio']);?></textarea>
    <div id="file_attachments_section">
      <label for="upload_file_button"><?php _e('Files','conf_scheduler')?>:</label>
    	<button id="upload_file_button" class="button"><?php _e( 'Add Files' ); ?></button>
    	<input type='hidden' name='file_attachments_id' id='file_attachments_id' value='<?php echo esc_attr($values['file_attachments_id']);?>'/>
      <div id="file_attachments"></div>
    </div>
    <?php
  }

  function workshop_side_meta_box_html() {
    global $post;
    wp_enqueue_script('select2v4-js');
    wp_enqueue_style('select2v4-css');

    // Get values for fields for this workshop
    $fields = array('workshop_id', 'location', 'presenter', 'limit', 'registered_participants');
    $values = array();

    foreach ($fields as $field) {
      $values[$field] = get_post_meta( $post->ID, $field, true );
    }

    $values['registered_participants'] = $values['registered_participants'] == '' ? 0 : $values['registered_participants'];


    $values['session'] = '';
    $workshop_session = get_the_terms($post->ID, 'conf_sessions');
    if($workshop_session) $values['session'] = $workshop_session[0]->term_id;

    $sessions = get_terms( array(
      'taxonomy'=> 'conf_sessions',
      'hide_empty' => false,
    ));

    // add start datetime data to sessions
    $sessions = array_map(
        function($obj){
          $obj->start = strtotime(get_term_meta($obj->term_id, 'start', true));
          return $obj;
        },
        $sessions);

    // sort sessions by datetime
    usort($sessions, function ($a, $b) {
        if ($a->start == $b->start) {
            return 0;
        }
        return ($a->start < $b->start) ? -1 : 1;
    });
    ?>
    <label for="workshop_id"><?php _e('Workshop ID','conf_scheduler')?>:</label>
    <input type='text' name='workshop_id' id='workshop_id' value='<?php echo esc_attr($values['workshop_id']);?>'/>

    <label for="location"><?php _e('Location','conf_scheduler')?>:</label>
    <input type='text' name='location' id='location' value='<?php echo esc_attr($values['location']);?>'/>

    <label for="presenter"><?php _e('Presenter','conf_scheduler')?>:</label>
    <input type='text' name='presenter' id='presenter' value='<?php echo esc_attr($values['presenter']);?>'/>

    <label for="limit"><?php _e('Participant Limit','conf_scheduler')?>:</label>
    <input type='number' name='limit' id='limit' value='<?php echo esc_attr($values['limit']);?>'/>

    <?php if ($this->options['allow_reg']):?>
    <a class="button" id="list_registered">List</a><label for="registered_participants"><?php _e('Registered Participants','conf_scheduler')?>:</label>
    <div id="registered"><?php echo $values['registered_participants'];?></div>
    <?php endif; ?>

    <div id="add_session"><a href="<?php echo admin_url('edit-tags.php?taxonomy=conf_sessions&post_type=conf_workshop');?>"><?php _e('+ Add Session', 'conf_scheduler');?></a></div><label for="session"><?php _e('Session','conf_scheduler')?>:</label>
    <select name='session' id='session' >
      <?php foreach ($sessions as $session) {
        $selected = intval($values['session']) == $session->term_id ? 'selected' : '';
        $n = $session->name . ' (' . date('j-M g:ia',$session->start).')';
        echo "<option value='{$session->term_id}' $selected>$n</option>";
      } ?>
    </select>

    <?php

  }

  function save_workshop_meta( $post_id, $post ) {
  	// Check we should save data (capability, nonce, not revision)
  	if ( ! current_user_can( 'edit_post', $post_id ) ) {
  		return $post_id;
  	}
  	if ( ! isset($_POST['conf_scheduler']) || ! wp_verify_nonce( $_POST['conf_scheduler'], basename(__FILE__) ) ) {
  		return $post_id;
  	}
    if ( 'revision' === $post->post_type ) return;

    // Validate attachment ids
    $values['file_attachments_id'] = implode(',', array_map(function($e){if(absint($e) > 0) return absint($e); }, explode(',', $values['file_attachments_id'])));
    update_post_meta( $post_id, 'file_attachments_id', $_POST['file_attachments_id'] );

    // Validate and sanitize text fields
  	update_post_meta( $post_id, 'presenter_bio', sanitize_text_field($_POST['presenter_bio']) );
    update_post_meta( $post_id, 'workshop_id', sanitize_text_field($_POST['workshop_id']) );
    update_post_meta( $post_id, 'location', sanitize_text_field($_POST['location']) );
    update_post_meta( $post_id, 'presenter', sanitize_text_field($_POST['presenter']) );

    // Validate and sanitize number fields
    $limit = absint(sanitize_text_field($_POST['limit']));
    $limit = $limit > 0 ? $limit : '';
    update_post_meta( $post_id, 'limit', $limit );

    $session = absint(sanitize_text_field($_POST['session']));
    $session_check = get_term($session, 'conf_sessions'); // returns null if not a valid session, uses term cache

    if(isset($session_check)) wp_set_object_terms( $post_id, $session, 'conf_sessions', false);

  }

  function save_session_meta( $term_id ) {
  	// Check we should save data (capability, nonce, not revision)
  	if ( ! current_user_can( 'edit_term', $term_id ) ) {
  		return $term_id;
  	}
  	if ( ! wp_verify_nonce( $_POST['conf_scheduler_sessions'], basename(__FILE__) ) ) {
  		return $term_id;
  	}

    // Sanitize, validate and format fields
    $start = strtotime(sanitize_text_field($_POST['start']));
    $start = $start !== false ? date('Y-m-d H:i',$start) : '';

    $length = intval(sanitize_text_field($_POST['length']));
    $length = $length !== 0 ? $length : '';

    $collapse = sanitize_text_field($_POST['collapse']);
    $collapse = $collapse == 'collapse' ? 'collapse' : 'seperate';

  	update_term_meta( $term_id, 'start', $start );
    update_term_meta( $term_id, 'length',  $length);
    update_term_meta( $term_id, 'collapse', $collapse );

  }

  function load_admin_style($hook) {
    if($hook != 'edit.php') { return; }
    wp_enqueue_style( 'conf_scheduler_admin_css', plugins_url('css/conf_scheduler_admin.css', __FILE__) );
  }

  function conf_scheduler_info_html() {
    $screen = get_current_screen();
    if ('edit-conf_workshop' == $screen->id):
    ?>
    <div id="conf-sidebar-wrap" style="display: none;">
      <div class="conf-sidebar">
      		<div class="content">
            <img class="logo" src="<?php echo plugins_url('images/logo.png', __FILE__);?>" height="34"/>
            <h2>Conference Scheduler</h2>
            <p class="subhead">Developed by: <a href="http://myceliumdesign.ca/conf_scheduler/">Shane Warner</a></p>
            <p><a href="http://myceliumdesign.ca/conf_scheduler/">Documentation</a></p>
            <p>This plugin was developed as a volunteer for a conference I was involved in. If you find it useful, please consider making a donation to help cover development costs.</p>
            <form action="https://www.paypal.com/cgi-bin/webscr" method="post" target="_top">
              <input type="hidden" name="cmd" value="_s-xclick">
              <input type="hidden" name="hosted_button_id" value="6XKHMYQLTMCZJ">
              <input type="hidden" name="amount" value="25">
              <input type="image" src="https://www.paypalobjects.com/en_US/i/btn/btn_donate_SM.gif" border="0" name="submit" alt="PayPal - The safer, easier way to pay online!">
              <img alt="" border="0" src="https://www.paypalobjects.com/en_US/i/scr/pixel.gif" width="1" height="1">
            </form>
          </div>
      		<div class="footer">
      			<p><?php _e('Thanks for using Conference Scheduler!','conf_scheduler' ); ?></p>
      		</div>
      </div>
    </div>

    <script type="text/javascript">
      (function($){
      	$('#posts-filter').wrap('<div class="conf-content" />');
      	$('#posts-filter').after( $('#conf-sidebar-wrap').html() );
      })(jQuery);
      </script>
    <?php
    endif;
  }

	// add new columns for session custom fields on admin
	function sessions_custom_columns_head($defaults) {
	    $defaults['conf_start']  = 'Start';
	    $defaults['conf_length'] = 'Length';
			unset($defaults['description']);
	    return $defaults;
	}

	function sessions_custom_columns_content($content,$column_name,$term_id) {
    $start = strtotime(get_term_meta( $term_id, 'start', true ));
    $length = get_term_meta( $term_id, 'length', true );

		if ($column_name == 'conf_start') {
        echo date('d-M-Y',$start).'<br/>'.date('g:ia',$start);
    }
    if ($column_name == 'conf_length') {
        echo $length.'min';
    }
	}

	function sessions_sortable_columns( $columns ) {
	    $columns['conf_start'] = 'conf_start';
			$columns['conf_length'] = 'conf_length';

	    return $columns;
	}

  function workshops_custom_columns_head($defaults) {
    if ($this->options['allow_reg']) {
	    $defaults['registered']  = 'Registered';
    }
    unset($defaults['date']);
    $defaults['taxonomy-conf_sessions'] = 'Session';
    return $defaults;
	}

	function workshops_custom_columns_content($column_name,$workshop_id) {

    if ($this->options['allow_reg'] && $column_name == 'registered') {
      $reg = get_post_meta($workshop_id, 'registered_participants',true);
      $reg = $reg == '' ? 0 : $reg;
      $limit = get_post_meta($workshop_id,'limit',true);
      echo $reg;
      if ($limit != '') echo '/'.$limit;
    }
	}

  function workshop_sortable_columns( $columns ) {
      $columns['taxonomy-conf_sessions'] = 'conf_sessions';
      $columns['registered'] = 'registered';

      return $columns;
  }

	function sessions_admin_orderby( $query ) {
	    if( ! is_admin() )
	        return;

        $orderby = ( isset( $_GET['orderby'] ) ) ? trim( sanitize_text_field( $_GET['orderby'] ) ) : '';

		    if( 'conf_start' == $orderby ) {
		        $query->query_vars['meta_key'] = 'start';
		        $query->query_vars['orderby'] = 'meta_value';
            $query->meta_query->parse_query_vars( $query->query_vars );
		    }
				if( 'conf_length' == $orderby ) {
		        $query->query_vars['meta_key'] = 'length';
		        $query->query_vars['orderby'] = 'meta_value_num';
            $query->meta_query->parse_query_vars( $query->query_vars );
		    }
	}

  function workshops_admin_orderby( $query ) {
      if ( ! is_admin() || ! $query->is_main_query() ) {
          return $pieces;
      }
      /*global $wpdb;
      $where_intro = $pieces[ 'where' ] != '' ? ' AND' : '';
      if ( ( $orderby = $query->get( 'orderby' ) ) == 'conf_sessions' ) {
          if ( ( $order = strtoupper( $query->get( 'order' ) ) ) != 'DESC' ) $order = 'ASC';
          $pieces[ 'join' ] .= ' LEFT JOIN ' . $wpdb->term_relationships . ' AS tr ON ' . $wpdb->posts . '.ID = tr.object_id'
              . ' LEFT JOIN ' . $wpdb->term_taxonomy . ' AS tt ON tr.term_taxonomy_id = tt.term_taxonomy_id'
              . ' LEFT JOIN ' . $wpdb->terms . ' AS tm ON tt.term_id = tm.term_id';
          $pieces[ 'fields' ] .= ', tm.name AS ' . $orderby;
          $pieces[ 'where' ] .=  $where_intro . " tt.taxonomy = '".$orderby."' ";
          $pieces[ 'orderby' ] = $orderby . ' ' . $order . ', ' . $wpdb->posts . '.post_title ASC';
      }*/

			if( ( $orderby = $query->get( 'orderby' ) ) == 'registered' ) {
	        //$query->query_vars['meta_key'] = 'registered_participants';
          //$query->query_vars['meta_compare'] = 'registered_participants';
          $query->query_vars['meta_query'] = array(
        		'relation' => 'OR',
        		'exists' => array(
        			'key'     => 'registered_participants',
        			'compare' => 'EXISTS',
        		),
        		'nexists' => array(
        			'key'     => 'registered_participants',
        			'compare' => 'NOT EXISTS',
        		),
        	);
	        $query->query_vars['orderby'] = array(
              'exists' => $query->get( 'order' )
          );
        //  $query->meta_query->parse_query_vars( $query->query_vars );

	    }

  }

	function render_schedule($atts) {
		// load scripts and styles
		wp_enqueue_style('conf_scheduler');
		wp_enqueue_style( 'fontawesome');
		wp_enqueue_script('isotope');
		wp_enqueue_script('js-cookie');

    // localize the js for AJAX
    $ajax_url = admin_url( 'admin-ajax.php' );
    wp_localize_script('conf_scheduler', 'ajax_url', $ajax_url );
		wp_enqueue_script('conf_scheduler');

		$output = '<div class="conf_scheduler">';

    $args = array( 'taxonomy'=> 'conf_sessions', 'hide_empty' => false );
		$sessions = get_terms( $args );

		// check for sessions to collapse
		foreach ($sessions as &$session) {
			if (get_term_meta($session->term_id, 'collapse', true) == 'collapse') {
				$session->merge_children = true;
			}

      // add custom data to session object
      $session->start = strtotime(get_term_meta($session->term_id, 'start', true));
      $session->length = get_term_meta($session->term_id, 'length', true);
		}

    // Filter sessions if requested
    $topParent = 0; // Defualt to show all sessions (starting with parent=>0 which is no parent)
    $session_tree = array();
    if($atts['id'] && intval($atts['id']) > 0 )
      $topParent = intval($atts['id']);

    if($atts['session']) {
      $parents = get_terms( array( 'taxonomy'=> 'conf_sessions', 'slug'=>$atts['session'], 'hide_empty' => false ) );
      $topParent = -1; // default to -1 to show no results
      foreach ($parents as $parent) {
        if ($parent->slug == $atts['session']) $topParent = $parent->term_id;
      }
    }


		$this->sort_terms_hierarchicaly($sessions, $session_tree, 'datetime', $topParent);

    if($topParent != 0 ) {
      $parent_key = false;
      foreach($sessions as $key => $session) {
        if($session->term_id == $topParent) {
          $parent_key = $key;
        }
      }

      if($parent_key !== false) {
        $parent = $sessions[$parent_key];
        $parent->parent = 0; // make it top so date shows
        $parent->children = $session_tree;
        $session_tree = array($parent);
      } else {
        $session_tree = array();
      }
    }

		// Search/filter options
		$output .= '<div class="filter"><span>'.__('Search','conf_scheduler').':</span>';

		$streams = get_terms( array(
			'taxonomy'=> 'conf_streams',
			'hide_empty' => false
		));
		if ($streams) {
			$stream_options = '<option value="">'.__('All Streams','conf_scheduler').'</option>';
			foreach ($streams as $stream ) {
					$stream_options .= '<option value="'.$stream->slug.'">'.$stream->name.'</option>';
			}
			$output .= '<select id="stream_filter">'.$stream_options.'</select>';
		}

    $areas = get_terms( array(
			'taxonomy'=> 'conf_areas',
			'hide_empty' => false
		));
		if ($areas) {
			$area_options = '<option value="">'.__('All Areas','conf_scheduler').'</option>';
			foreach ($areas as $area ) {
					$area_options .= '<option value="'.$area->slug.'">'.$area->name.'</option>';
			}
			$output .= '<select id="area_filter">'.$area_options.'</select>';
		}

		$output .= '<input type="text" id="workshop_search" placeholder="'.__('Search','conf_scheduler').'..."/>';
		$output .= '<button id="toggle_all">'.__('Show/Hide All','conf_scheduler').'</button>';
		$output .= '<button id="my_picks">'.__('Show My Picks','conf_scheduler').'</button></div>';

    if ($session_tree) {
      $output .= $this->render_session($session_tree);
    } else {
      $output .= 'No sessions found.';
    }

		$output .= '</div>'; // close .conf_scheduler

		return $output;
	}

	function render_session($session_tree) {
		$output = '';
		foreach ($session_tree as $session_id => $session) {
			if ($session->parent == 0) {
				if (date('Y-m-d', $session->start) != $this->last_date) {
					if ($this->last_date != "0") $output .= '</div>'; // close day
					$output .= '<div class="conference_day"><h3>'.date("l jS M Y", $session->start).'</h3>';
					$this->last_date = date('Y-m-d', $session->start);
				}
			}

			$output .= '<div class="session">';
			$output .= '<h3>'.date('g:i', $session->start).' - '.date('g:ia', ($session->start + $session->length *60) ).'</h3><div class="workshops">';

			// render session workshops
			$output .= $this->render_session_workshops($session);

			if($session->merge_children && isset($session->children)) {
				// add child sessions as split boxes
				$output .= $this->render_session_merged_workshops($session->children);
				unset($session->children);
			}
			$output .= '</div>'; // close workshops

			if(isset($session->children)) {
				$output .= $this->render_session($session->children);
			}

			$output .= '</div>'; // close session
		}

		if ($this->last_date == "0") $output .= '</div>'; // close last day

		return $output;
	}

	function render_session_workshops($session) {
		$output = '';

		$workshops = new WP_Query(array('post_type' => 'conf_workshop', 'tax_query' => array( array( 'taxonomy' => 'conf_sessions', 'field' => 'term_id', 'terms' => $session->term_id , 'include_children'=>false)), 'nopaging'=>true));
    if($workshops->have_posts()) {
  		foreach ($workshops->posts as $workshop) {
  			$output .= $this->render_workshop($workshop, $session);
  		}
    } else {
      $output .= '<p class="no_workshops">'.__('No workshops in this session.', 'conf_scheduler').'</p>';
    }
		return $output;
	}

	function render_workshop($workshop, $session, $block = 'conf_block') {
		$output = '';
    $classes = array('workshop');
    $classes[] = esc_attr($block);

		// format session details
		$session_text = '<div class="session '.$session->slug.'">'.date('D',$session->start).' '.date('g:i', $session->start).' - '.date('g:ia', ($session->start + $session->length *60) ).'</div>';

		// generate edit link
		if (current_user_can( 'edit_post' , $workshop->ID )) {
			$edit_link = '<a class="edit_link" title="Edit Workshop" href="'.get_edit_post_link( $workshop->ID, 'link' ).'"></a>';
		}else { $edit_link = ''; }

    $tags = array();

		// get & format streams
		$streams = get_the_terms($workshop->ID,'conf_streams');
		if ($streams) {
			foreach ($streams as $stream) {
				$tags[] = '<span class="stream '.$stream->slug.'">'.$stream->name. '</span>';
				$classes[] = 'stream_'.$stream->slug;
			}
		}

    // get & format areas
		$areas = get_the_terms($workshop->ID,'conf_areas');
		if ($areas) {
			foreach ($areas as $area) {
				$tags[] = '<span class="area '.$area->slug.'">'.$area->name. '</span>';
				$classes[] = 'area_'.$area->slug;
			}
		}

    $tags_html = $tags ? '<div class="tags">'.implode(', ',$tags).'</div>' : '';

		$details = array();
    $details['workshop_id'] = get_post_meta($workshop->ID,'workshop_id', true);
    $details['location'] = get_post_meta($workshop->ID,'location', true);
    $details['presenter'] = get_post_meta($workshop->ID,'presenter', true);
    $details['limit'] = get_post_meta($workshop->ID,'limit', true);
    $details['session'] = get_post_meta($workshop->ID,'session', true);
    $details['presenter_bio'] = get_post_meta($workshop->ID,'presenter_bio', true);
    $files_data = get_post_meta($workshop->ID,'file_attachments_id', true);

    $presenter = '';
    if (isset($details['presenter']) && !empty($details['presenter'])) {
      $presenter = '<span class="presenter">'.$details['presenter'].'</span>';
    }

    $location = '';
    if (isset($details['location']) && !empty($details['location'])) {
      $location = '<span class="location">'.$details['location'].'</span>';
    }

    $files = '';
    if ($files_data) {
      $file_ids = array_map('intval', explode(',', $files_data));
      $files = '<div class="files"><span>'.__('Documents','conf_scheduler').'</span>';
      foreach($file_ids as $file_id) {
        $files .= '<a class="file" href="'.wp_get_attachment_url($file_id).'">'.get_the_title($file_id)."</a> <small>(".size_format( filesize( get_attached_file($file_id))).")</small><br/>";
      }
      $files .= '</div>';
    }

    $limit = '';
    if ($details['limit'] && $details['limit'] > 0) {
      $limit = '<div class="limit" title="Participant limit">'.esc_html($details['limit']).' max</div>';
    }

    // Register button
    $register = '';
    if($this->options['allow_reg']) {
      $details['registered_participants'] = get_post_meta($workshop->ID,'registered_participants', true);
      if ($details['registered_participants'] == '') $details['registered_participants'] = 0;

      $user_id = get_current_user_id();
      $user_registered = get_user_meta($user_id,'conf_scheduler_registered_workshops', true);
      if ( in_array($workshop->ID, $user_registered) ) {
        $register = '<button class="register" title="Unregister for this workshop"><i class="fa fa-check"></i>Registered</button>';
        $classes[] = 'registered';
      } elseif ($details['limit'] != '' && ($details['limit'] <= 0 || $details['registered_participants'] >= $details['limit'] )) {
        $register = '<button class="register" disabled>Workshop Full</button>';
        $classes[] = 'registration_full';
      } else {
        $register = '<button class="register" title="Register for this workshop">Register</button>';
      }
    }

    $image = get_the_post_thumbnail($workshop->ID, 'medium');
    $pick_title = esc_attr(__('Pick this workshop','conf_scheduler'));

    //escape output
    $escape = array('presenter','location','presenter_bio');
    foreach($escape as $field) {
      $details[$field] = esc_html($details[$field]);
    }
    $class_text = implode($classes,' ');
		$output .= <<< END
				<div class="$class_text" data-workshopID="{$workshop->ID}">
					<div class="favorite" title="$pick_title">
						<span class="fa-stack">
							<i class="fa fa-star fa-stack-1x"></i>
							<i class="fa fa-star-o fa-stack-1x"></i>
						</span>
					</div>
					<h4 class="title">$workshop->post_title</h4>
					<p class="quick_info">$presenter $location</p>
					<div class="details">
						<div class="description">$workshop->post_content</div>
            $image
						<div class="bio">{$details['presenter_bio']}</div>
            $files
					</div>
					<div class="data">
						$session_text
            $limit
						$tags_html
					</div>
          $register
					$edit_link
				</div>
END;
		return $output;
	}

	function render_session_merged_workshops( $sessions ) {
		$output = '';
		$workshops = array();
		$sessions = array_values($sessions);
		foreach ($sessions as $session ) {
			 $session_workshops = new WP_Query (array('post_type' => 'conf_workshop', 'tax_query' => array( array( 'taxonomy' => 'conf_sessions', 'field' => 'term_id', 'terms' => $session->term_id , 'include_children'=>false)), 'nopaging'=>true));
       $workshops[] = $session_workshops->posts;
		}

    if(count($workshops) > 0) {
  		$max_workshops = max(array_map( 'count', $workshops ));
  		for ($i = 0; $i < $max_workshops; $i++ ) {
  			$output .= '<div class="workshop_group conf_block">';
  			foreach ($workshops as $k => $session) {
  				if(isset($session[$i])) {
  					$output .= $this->render_workshop($session[$i], $sessions[$k], '');
  				} else {
  					$output .= '<div class="workshop placeholder"></div>';
  				}

  			}
  			$output .= '</div>';
  		}
    }

		return $output;
	}

	function sort_terms_hierarchicaly(Array &$cats, Array &$into, $sortby, $parentId = 0) {
      if($parentId != -1) {
        foreach ($cats as $i => $cat) {
  	        if ($cat->parent == $parentId) {
  	            $into[$cat->term_id] = $cat;
  	            unset($cats[$i]);
  	        }
  	    }
      }

			// sort terms by date/start_time
      if($sortby == 'datetime') {
  			uasort($into, function ($a, $b) {
  			    if ($a->start == $b->start) {
  			        return 0;
  			    }
  			    return ($a->start < $b->start) ? -1 : 1;
  			});
      }

	    foreach ($into as $topCat) {
	        $topCat->children = array();
	        $this->sort_terms_hierarchicaly($cats, $topCat->children, $sortby, $topCat->term_id);
	    }
	}

  function register_for_workshop ($workshop_id = null, $user_id = null) {
    if ($workshop_id == null) {
      return false; //
    }
    if ($user_id == null) {
      $user_id = get_current_user_id(); // default to current user
    }
    if (!$user_id) { return false; } // fail if there is no valid user

    $reg_workshops = get_user_meta($user_id, 'conf_scheduler_registered_workshops', true);
    $reg_participants = get_post_meta($workshop_id, 'registered_participants', true);
    if($reg_participants == '') $reg_participants = 0;
    $reg_participants++;
    $participant_limit = get_post_meta($workshop_id, 'limit', true);
    if($participant_limit == '') $participant_limit = 10000;

    if (is_array($reg_workshops) && !in_array($workshop_id, $reg_workshops) && $reg_participants <= $participant_limit) {
      $reg_workshops[] = $workshop_id;
    } elseif (!is_array($reg_workshops) && $participant_limit > 0) {
      $reg_workshops = array($workshop_id);
    } elseif ($reg_participants > $participant_limit) {
      return 'limit'; // Can't add
    } else {
      return false;
    }

    update_user_meta( $user_id, 'conf_scheduler_registered_workshops', $reg_workshops );
    update_post_meta( $workshop_id, 'registered_participants', $reg_participants );

    return $reg_participants; // return new total of registered attendees
  }

  function unregister_for_workshop ($workshop_id = null, $user_id = null) {
    if ($workshop_id == null) {
      return false; //
    }
    if ($user_id == null) {
      $user_id = get_current_user_id(); // default to current user
    }
    if (!$user_id) { return false; } // fail if there is no valid user

    $reg_workshops = get_user_meta($user_id, 'conf_scheduler_registered_workshops', true);

    if (is_array($reg_workshops) && in_array($workshop_id, $reg_workshops)) {
      $key = array_search($workshop_id, $reg_workshops);
      unset($reg_workshops[$key]);
    } else {
      return false;
    }

    update_user_meta( $user_id, 'conf_scheduler_registered_workshops', $reg_workshops );

    // update the postmeta for the workshop
    $reg_participants = get_post_meta($workshop_id, 'registered_participants', true);
    if($reg_participants == '') $reg_participants = 0;
    $reg_participants--;
    if($reg_participants < 0) $reg_participants = 0;
    update_post_meta( $workshop_id, 'registered_participants', $reg_participants );

    return $reg_participants; // return new total of registered attendees
  }

  function ajax_registration_callback() {
    if( ! isset( $_POST ) || empty( $_POST ) || ! is_user_logged_in() ) {
      header( 'HTTP/1.1 400 Empty POST Values' );
      echo '';
      wp_die();
    }

    $workshop_id = intval($_POST['workshop_id']);
    $return = array("req_id" =>$workshop_id);
    if($workshop_id > 0) {
      //register for the workshop
      $return['result'] = $this->register_for_workshop($workshop_id);
      $return['action'] = 'register';
    }

    if($workshop_id < 0) {
      // unregister for workshop
      $real_id = absint($workshop_id);
      $return['result'] = $this->unregister_for_workshop($real_id);
      $return['action'] = 'unregister';
    }

    if ($return['result'] === false) {
      $return['error'] = true;
    }

    echo json_encode($return);

    wp_die();
  }

  function get_registered_participant_ids($workshop_id) {
    global $wpdb;

    $usermeta = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}usermeta WHERE meta_key = 'conf_scheduler_registered_workshops'", OBJECT );
    if($usermeta) {
      $users = array();
      foreach ($usermeta as $u) {
        $users[$u->user_id] = unserialize($u->meta_value);
      }
      $reg_users = array_filter($users, function ($user_reg) use ($workshop_id) { return array_search($workshop_id, $user_reg) !== false ? true : false; } );
    } else {
      $reg_users = array();
    }
    return array_keys($reg_users);
  }

  function list_registered_participants() {
    if( ! isset( $_POST ) || empty( $_POST ) || ! is_user_logged_in() ) {
      header( 'HTTP/1.1 400 Empty POST Values' );
      echo '';
      wp_die();
    }

    $workshop_id = intval($_POST['workshop_id']);
    $user_ids = $this->get_registered_participant_ids($workshop_id);

    foreach ($user_ids as $user) {
      $data = get_userdata($user);
      if ($data->first_name != '' && $data->last_name != '') {
        echo $data->first_name .' '.$data->last_name;
      } else {
        echo $data->display_name;
      }
      echo '<br/>';
    }

    wp_die();

  }

  function options_menu_item() {
    add_submenu_page(
      'edit.php?post_type=conf_workshop',
      'Conference Scheduler Settings',
      'Settings',
      'manage_options',
      'conf_scheduler_options',
      array($this,'conf_scheduler_options')
    );
  }

  function conf_scheduler_options() {
    if (!current_user_can('manage_options')) {
        wp_die('Unauthorized user');
    }

    if (isset($_POST['conf_scheduler_options_nonce'])) {
      if (!wp_verify_nonce( $_POST['conf_scheduler_options_nonce'], 'conf_scheduler_options' )) {
          wp_die('Nonce verification failed');
      } else {
        do_action('conf_scheduler_process_options');
        $options = array();
        $options['allow_reg'] = $_POST['allow_reg'] == 1 ? 1 : 0;
        update_option('conf_scheduler_options', $options);
      }
    } else {
      $options = get_option('conf_scheduler_options', array());
    }

    include 'templates/options.php';
  }

} // close class

function conf_scheduler() {
	global $conf_scheduler;

	if( !isset($conf_scheduler) ) {
		$conf_scheduler = new conf_scheduler();
		$conf_scheduler->initialize();
	}

	return $conf_scheduler;
}

// initialize
conf_scheduler();

endif; // class_exists check

?>
