/*
Conference Scheduler
Version: 1.0
License: GNU General Public License v2 or later
License URI: http://www.gnu.org/licenses/gpl-2.0.html
*/

// Setup master JS var
var conf_scheduler = { search: '', myPicks: new Array()};

jQuery(document).ready(function($){

  var _workshops = $('.conf_block');

  $('.conf_block').attr('data-order','5');

  // load saved picks
  if (Cookies.get('conf_scheduler_picks')) conf_scheduler.myPicks = JSON.parse(Cookies.get('conf_scheduler_picks'));
  if (conf_scheduler.myPicks.length > 0 ) {
    conf_scheduler.myPicks.forEach(function(workshopID) {
      $('.workshop[data-workshopID='+workshopID+']').addClass('picked').closest('.conf_block').attr('data-order', '1');
    });
  }

  var _layouts = $('.workshops').each(function() {
    var _sizer = $(this).find('.conf_block');
    $(this).isotope({
      // options
      itemSelector: '.conf_block',
      percentPosition: true,
      masonry: {
        columnWidth: _sizer[0]
      },
      getSortData: {
        order: '[data-order]',
      }
    });
  });

  if (conf_scheduler.myPicks.length > 0 ) _layouts.isotope({ sortBy: 'order'});


  // Filter function
  conf_scheduler.filterFunc = function() {
    var stream = $('#stream_filter').length > 0 ? $('#stream_filter').val() : '';
    var area = $('#area_filter').length > 0 ? $('#area_filter').val() : '';
    var search = $('#workshop_search').val().toLowerCase();
    if (stream == '' && area == '' && search == '') { return true; }

    var _workshop = $(this);
    if ($(this).hasClass('workshop_group')) {
       _workshop = $(this).find('.workshop');
    }

    var found = false;

    _workshop.each(function() {
      if (stream != '') { inStream = $(this).hasClass('stream_'+stream) } else { inStream = true; }
      if (area != '') { inArea = $(this).hasClass('area_'+area) } else { inArea = true; }
      if (search != '') {
        inSearch = ($(this).find('.title').text().toLowerCase().indexOf(search) != -1 || $(this).find('.presenter').text().toLowerCase().indexOf(search) != -1)
      } else { inSearch = true; }
      found = found || (inStream && inArea && inSearch);
      return true;
    });

    return found;
  };

  // Live search function
  conf_scheduler.liveSearcher = function() {
    if (window.liveSearch) clearTimeout(window.liveSearch);
    window.liveSearch = setTimeout(function(){
      $('.conference_day').addClass('open');
      $('.session').addClass('open');
      $('#my_picks').removeClass('show');
      if ($('#workshop_search').val().toLowerCase() != '') {
        jQuery('.session h3').hide();
      } else {
        jQuery('.session h3').show();
      }
      _layouts.isotope({ filter: conf_scheduler.filterFunc})
    }, 500);
  };

  // Trigger events
  $('#stream_filter').change(function(e){
    $('.conference_day').addClass('open');
    $('.session').addClass('open');
    $('#my_picks').removeClass('show');
    _layouts.isotope({ filter: conf_scheduler.filterFunc });
  });
  $('#area_filter').change(function(e){
    $('.conference_day').addClass('open');
    $('.session').addClass('open');
    $('#my_picks').removeClass('show');
    _layouts.isotope({ filter: conf_scheduler.filterFunc });
  });
  $('#workshop_search').keyup(conf_scheduler.liveSearcher);
  $('#workshop_search').click(conf_scheduler.liveSearcher);

  // Expand workshop details
  $('.workshop').click(function(){
    $(this).find('.details').toggle();
    $(this).toggleClass("open");
    if($(this).parent().hasClass('workshop_group') ){
      $(this).parent().parent().isotope('layout');
    }else{ $(this).parent().isotope('layout'); }
  }).children('.favorite').click(function(e) {
    return false;
  });

  $('.workshop .file').click(function(e) {
    e.stopPropagation();
  }); // prevent closing of workshop when viewing attached documents

  // Fold days  - delegated to parent elem becuase elems might be initially hidden
  $('body').on('click','.conference_day > h3', function(){
    $(this).parent().toggleClass('open');
  });

  // Fold sessions - delegated to parent elem becuase elems are initially hidden
  $('body').on('click', '.session > h3', function(){
    $(this).parent().toggleClass('open');
    $(this).siblings('.workshops').isotope('layout');
  });

  // show/hide all workshops
  $('#toggle_all').click(function(){
    $('#my_picks').removeClass('show');
    $('#stream_filter option').first().prop('selected',true).change(function(){ return false; });
    $('#area_filter option').first().prop('selected',true).change(function(){ return false; }); // reset area filter and prevent it's function
    $('#workshop_search').val(''); // reset search
    if ($('.workshops:visible').length > 0) {
      _layouts.isotope({ filter: '*'});
      $('.conference_day').removeClass('open');
      $('.session').removeClass('open');
    }else{
      $('.conference_day').addClass('open');
      $('.session').addClass('open');
      _layouts.isotope({ filter: '*'});
    }

  });

  // Show only My Picks
  $('#my_picks').click(function(){
    $(this).toggleClass('show');
    if( $(this).hasClass('show') ) {
      $('#stream_filter option').first().prop('selected',true).change(function(){ return false; }); // reset stream filter and prevent it's function
      $('#area_filter option').first().prop('selected',true).change(function(){ return false; }); // reset area filter and prevent it's function
      $('#workshop_search').val(''); // reset search
      $('.conference_day').addClass('open');
      $('.session').addClass('open');
      _layouts.isotope({ filter: function() { return ( $(this).hasClass('picked') || $(this).find('.picked').length ) }});
    } else {
      _layouts.isotope({ filter: '*'});

    }
  });

  // Pick a workshop
  $('.workshop .favorite').click(function(){
    var _workshop = $(this).parent().toggleClass('picked');
    if (_workshop.hasClass('picked') ) {
      conf_scheduler.myPicks.push(parseInt(_workshop.attr('data-workshopID')));
    } else {
      conf_scheduler.myPicks = conf_scheduler.myPicks.filter(function(item) {
          return item !== parseInt(_workshop.attr('data-workshopID'));
      });
    }

    Cookies.set('conf_scheduler_picks', conf_scheduler.myPicks);

    // promote this block and re-sort
    var _block = $(this).closest('.conf_block');
    if ( _block.hasClass('picked') || _block.find('.picked').length ) {
      _block.attr('data-order','1');
    } else { _block.attr('data-order','5'); }

    _layouts.isotope('updateSortData').isotope({ sortBy: 'order'});
  });



  var register = function ($workshop, resp) {
    if(!resp.error && resp.result != 'limit') {
      $workshop.addClass('registered');
      $workshop.find('.register').text('Registered').attr('title', "Unregister for this workshop").prepend('<i class="fa fa-check"></i>');
      $workshop.find('.favorite').click(); // favorite the workshop
    } else if (resp.result == 'limit') {
      $workshop.addClass('registration_full');
      $workshop.find('.register').text('Workshop Full').prop('disabled',true);
    } else {

    };
  }

  var unregister = function( $workshop, resp ) {
    if(!resp.error) {
      $workshop.removeClass('registered');
      $workshop.find('.register').text('Register').attr('title', "Register for this workshop");
      $workshop.find('.favorite').click(); // unfavorite the workshop
    } else {
      $workshop.find('.register').prepend('<i class="fa fa-check"></i>');
    }
  }

  var processAJAXregister = function( results ) {
    var resp = JSON.parse(results);
    var $this = $(this);
    var $workshop = $this.parents('.workshop');
    $this.find('.fa-spinner').remove();
    if($workshop.hasClass('registered')) {
      unregister($workshop, resp);
    } else {
      register($workshop, resp);
    }
  }

  // Handle AJAX workshop registration
  $('.workshop .register').click(function(e) {
    e.stopPropagation(); // stop workshop from collapsing
    $this = $(this);
    $workshop = $this.parents('.workshop');
    workshopID = $workshop.data('workshopid');
    if ( $workshop.hasClass('registered') ) {
      workshopID = workshopID * -1 // flip int to trigger unregister
      $this.find('.fa-check').remove();
    }
    $this.prepend('<i class="fa fa-spin fa-spinner"></i>');
    $.ajax( {
        url : ajax_url,
        type: 'POST',
        data: {
            action  : 'conf_scheduler_register',
            'workshop_id': workshopID,
        },
        context: this,
        success: processAJAXregister,
        error: function( data ) {
            $this.find('.fa-spinner').remove();
        }
    } );
  });
});
